"use strict";

var _ava = _interopRequireDefault(require("ava"));

var _sinon = require("sinon");

var _departments = _interopRequireDefault(require("./departments"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const MOCK_ROOT_DEPARTMENT = Object.freeze({
  cs_domain_id: '0000-de00-4000-a000-000000000001',
  date_created: '2012-12-21T11:12:43Z',
  disable_personal_accounts: false,
  display_name: 'Mid Winter',
  enabled: true,
  id: 1,
  level: 0,
  name: 'winter-solstice'
});
const TRANSFORMED_ROOT_DEPARTMENT = Object.freeze({
  created: new Date('2012-12-21T11:12:43Z'),
  displayName: 'Mid Winter',
  enabled: true,
  id: 1,
  level: 0,
  name: 'winter-solstice',
  parent: null,
  settings: {
    projectOnly: false
  },
  status: 'enabled',
  uuid: '0000-de00-4000-a000-000000000001'
});
const MOCK_CHILD_DEPARTMENT = Object.freeze({
  ancestor: {
    id: 1
  },
  cs_domain_id: null,
  date_created: '2015-06-21T16:39:02Z',
  disable_personal_accounts: true,
  display_name: 'Mid Summer',
  enabled: false,
  id: 2,
  level: 0,
  name: 'summer-solstice'
});
const TRANSFORMED_CHILD_DEPARTMENT = Object.freeze({
  created: new Date('2015-06-21T16:39:02Z'),
  displayName: 'Mid Summer',
  enabled: false,
  id: 2,
  level: 0,
  name: 'summer-solstice',
  parent: {
    id: 1
  },
  settings: {
    projectOnly: true
  },
  status: 'disabled',
  uuid: null
});
const MOCK_DEPARTMENTS = Object.freeze([MOCK_ROOT_DEPARTMENT, MOCK_CHILD_DEPARTMENT]);
const TRANSFORMED_DEPARTMENTS = Object.freeze([TRANSFORMED_ROOT_DEPARTMENT, TRANSFORMED_CHILD_DEPARTMENT]);

function fakeRoutes(routes = []) {
  return async (method, command, {
    data,
    params
  } = {}) => {
    // eslint-disable-next-line no-restricted-syntax
    for (const route of routes) {
      if (method === route.method && command === route.command) {
        return route.fn({
          data,
          params
        });
      }
    }

    if (method === 'GET' && command === 'reseller') {
      return MOCK_DEPARTMENTS;
    }

    if (method === 'GET' && command === 'reseller/1') {
      return MOCK_ROOT_DEPARTMENT;
    }

    if (method === 'GET' && command === 'reseller/2') {
      return MOCK_CHILD_DEPARTMENT;
    }

    throw new Error('unreachable');
  };
}

(0, _ava.default)('list()', async t => {
  const call = _sinon.fake.resolves({
    objects: MOCK_DEPARTMENTS
  });

  const {
    list
  } = (0, _departments.default)(call);
  t.deepEqual((await list()), TRANSFORMED_DEPARTMENTS);
});
(0, _ava.default)('list(id)', async t => {
  const call = _sinon.fake.resolves(MOCK_ROOT_DEPARTMENT);

  const {
    list
  } = (0, _departments.default)(call);
  t.deepEqual((await list(2)), TRANSFORMED_ROOT_DEPARTMENT);
});
(0, _ava.default)('create()', async t => {
  const call = (0, _sinon.fake)(fakeRoutes([{
    method: 'POST',
    command: 'reseller',
    fn: () => ({
      id: 2
    })
  }]));
  const {
    create
  } = (0, _departments.default)(call);
  t.deepEqual((await create({
    name: 'summer-solstice',
    displayName: 'Mid Summer',
    parent: {
      uuid: '0000-de00-4000-a000-000000000001'
    }
  })), TRANSFORMED_CHILD_DEPARTMENT);
  t.true(call.calledWith('POST', 'reseller', {
    data: {
      cs_domain_name: 'summer-solstice',
      enabled: true,
      disable_personal_accounts: false,
      domain: 'summer-solstice',
      name: 'summer-solstice',
      display_name: 'Mid Summer',
      parent_domain_id: '0000-de00-4000-a000-000000000001'
    }
  }));
});
(0, _ava.default)('create() - defaults', async t => {
  const call = _sinon.fake.resolves('irrelevant');

  const {
    create
  } = (0, _departments.default)(call);
  await create({
    name: 'minimalist'
  }).catch(() => {});
  t.true(call.calledWith('POST', 'reseller', {
    data: {
      cs_domain_name: 'minimalist',
      enabled: true,
      disable_personal_accounts: false,
      domain: 'minimalist',
      name: 'minimalist',
      display_name: 'minimalist'
    }
  }));
});
(0, _ava.default)('update()', async t => {
  const clone = { ...MOCK_CHILD_DEPARTMENT
  };
  const call = (0, _sinon.fake)(fakeRoutes([{
    method: 'PUT',
    command: 'reseller/1',

    fn({
      data
    }) {
      Object.assign(clone, data);
      return 'irrelevant';
    }

  }, {
    method: 'GET',
    command: 'reseller/1',
    fn: () => clone
  }]));
  const {
    update
  } = (0, _departments.default)(call);
  t.deepEqual((await update(1, {
    displayName: 'New Name'
  })), { ...TRANSFORMED_CHILD_DEPARTMENT,
    displayName: 'New Name'
  });
});
(0, _ava.default)('delete()', async t => {
  const call = _sinon.fake.resolves('irrelevant');

  const {
    delete: remove
  } = (0, _departments.default)(call);
  t.is((await remove(1)), 1);
  t.true(call.calledWith('DELETE', 'reseller/1'));
});