"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = projects;

var _objects = require("../../utils/objects");

var _users = require("./users");

function into(project) {
  return {
    account: {
      id: project.account.id,
      username: project.account.username,
      uuid: project.account.uuid || null
    },
    accounts: project.accounts || [],
    created: new Date(project.date_created),
    department: {
      id: project.reseller.id,
      uuid: project.reseller.cs_domain_id || null
    },
    description: project.description,
    displayName: project.display_name || null,
    enabled: project.enabled,
    id: project.id,
    name: project.group_name,
    type: project.source,
    subscriptionNumber: project.subscription_number,
    productCode: project.product_code
  };
}

function intoCreateValues({
  name,
  department,
  description = '',
  displayName = name,
  subscriptionNumber,
  productCode,
  type
}) {
  return {
    group_name: name,
    display_name: displayName,
    description,
    reseller_id: department,
    subscription_number: subscriptionNumber,
    product_code: productCode,
    source: type
  };
}

function intoUpdateValues({
  displayName,
  description
}) {
  return (0, _objects.prune)({
    description,
    display_name: displayName
  });
}

function projects(call) {
  async function list(id) {
    if (id) {
      const project = await call('GET', `projects/${id}`);
      return into(project);
    }

    const {
      objects
    } = await call('GET', 'projects');
    return objects.map(into);
  }

  async function keys(id) {
    const {
      account: {
        username
      }
    } = await list(id);
    const result = await call('GET', 'account-keys/get_user_keys', {
      params: {
        username
      }
    });
    return {
      apiKey: result.api_public_key,
      secretKey: result.api_private_key
    };
  }

  async function create(values) {
    const data = intoCreateValues(values);
    const {
      id
    } = await call('POST', 'projects', {
      data
    });
    return list(id);
  }

  async function update(id, values) {
    const data = intoUpdateValues(values);
    await call('PUT', `projects/${id}`, {
      data
    });
    return list(id);
  }

  async function enable(id) {
    await call('POST', `projects/${id}/enable`);
    return list(id);
  }

  async function disable(id) {
    await call('POST', `projects/${id}/disable`);
    return list(id);
  }

  async function listUsers(id) {
    const users = await call('GET', `projects/${id}/users`);
    return users.map(_users.into);
  }

  async function addUser(id, username) {
    await call('POST', `projects/${id}/add_user`, {
      data: {
        username
      }
    });
    return list(id);
  }

  async function removeUser(id, username) {
    await call('POST', `projects/${id}/remove_user`, {
      data: {
        username
      }
    });
    return list(id);
  }

  async function remove(id) {
    await call('GET', `projects/${id}/delete`);
    return id;
  }

  return {
    list,
    keys,
    create,
    update,
    enable,
    disable,
    listUsers,
    addUser,
    removeUser,
    delete: remove
  };
}