"use strict";

var _ava = _interopRequireDefault(require("ava"));

var _sinon = require("sinon");

var _error = require("../../../service/error");

var _jobs = require("./jobs");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sleep(ms) {
  return new Promise(resolve => {
    global.setTimeout(resolve, ms);
  });
}

(0, _ava.default)('finish() calls ‘GET Jobs/<id>’', async t => {
  const call = _sinon.fake.resolves({
    state: 'done',
    jobId: 'some-id'
  }, 'some-id');

  await (0, _jobs.finish)(call, 'some-id');
  t.true(call.calledWith('GET', 'Jobs/some-id'));
});
(0, _ava.default)('finish() resolves on ‘done’', async t => {
  const job = {
    state: 'done',
    jobId: 'done-id'
  };
  t.is((await (0, _jobs.finish)(_sinon.fake.resolves(job), 'done-id')), job);
});
(0, _ava.default)('finish() rejects on ‘error’', async t => {
  const job = {
    state: 'error',
    stateDetails: 'Some error message',
    jobId: 'error-id'
  };
  const error = await t.throwsAsync((0, _jobs.finish)(_sinon.fake.resolves(job), 'error-id'), _error.ServiceError);
  t.is(error.code, 500);
  t.is(error.message, 'Some error message');
  t.is(error.details, job);
});
(0, _ava.default)('finish() times out if time passes timeout', async t => {
  const job = {
    action: 'create',
    state: 'ongoing',
    jobId: 'timeout-id',
    objectType: 'Snapshot',
    objectId: 'snapshot-id'
  };
  const finishing = (0, _jobs.finish)(_sinon.fake.resolves(job), 'timeout-id', () => {}, 10, 100);
  const error = await t.throwsAsync(finishing, _error.ServiceError);
  t.is(error.code, 408);
  t.is(error.message, 'job "create" for object "Snapshot" with id "snapshot-id" timed out after 0 seconds');
  t.is(error.details, job);
});
(0, _ava.default)('finish() pools while job is ongoing', async t => {
  const job = {
    state: 'ongoing',
    jobId: 'ongoing-id'
  };

  const call = _sinon.fake.resolves(job);

  (0, _jobs.finish)(call, 'ongoing-id', _sinon.fake.resolves(), 20, 10000);
  t.is(call.callCount, 1);
  await sleep(110); // call count should increase while job is running

  const currentCount = call.callCount;
  t.true(currentCount > 1);
  job.state = 'done';
  await sleep(50); // Only one more call needed.

  t.is(call.callCount, currentCount + 1);
});
(0, _ava.default)('finish() calls sideEffect while job is ongoing', async t => {
  const job = {
    state: 'ongoing',
    jobId: 'ongoing-id'
  };

  const call = _sinon.fake.resolves(job);

  const side = _sinon.fake.resolves();

  (0, _jobs.finish)(call, 'ongoing-id', side, 20, 10000);
  t.is(side.callCount, 0);
  await sleep(110); // call count should increase while job is running.

  const currentCount = side.callCount;
  t.true(currentCount > 0);
  job.state = 'done';
  await sleep(50); // call count should not increase after job has finished.

  t.is(side.callCount, currentCount);
});
(0, _ava.default)('finish() resolves when state becomes ‘done’', async t => {
  const job = {
    state: 'ongoing',
    jobId: 'ongoing-id'
  };

  const call = _sinon.fake.resolves(job);

  const done = _sinon.fake.resolves();

  const finishing = (0, _jobs.finish)(call, 'ongoing-id', _sinon.fake.resolves(), 20, 10000);
  finishing.then(done);
  t.false(done.calledOnce);
  await sleep(110);
  t.false(done.calledOnce);
  job.state = 'done';
  await sleep(50);
  t.true(done.calledOnce);
});
(0, _ava.default)('finish() sideEffect can be null', async t => {
  const job = {
    state: 'ongoing',
    jobId: 'ongoing-id'
  };

  const call = _sinon.fake.resolves(job);

  const done = _sinon.fake.resolves();

  const finishing = (0, _jobs.finish)(call, 'ongoing-id', null, 20, 10000);
  finishing.then(done);
  await sleep(110);
  job.state = 'done';
  await sleep(50);
  t.true(done.calledOnce);
});
(0, _ava.default)('finish() works with default values', async t => {
  const job = {
    state: 'ongoing',
    jobId: 'ongoing-id'
  };

  const call = _sinon.fake.resolves(job);

  const done = _sinon.fake.resolves();

  const finishing = (0, _jobs.finish)(call, 'ongoing-id');
  finishing.then(done); // Default interval is 500ms.

  await sleep(510); // Initial call + one poll.

  t.is(call.callCount, 2);
  job.state = 'done';
  await sleep(510);
  t.is(call.callCount, 3);
  t.true(done.calledOnce);
});