"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRequest = getRequest;
exports.handleResponse = handleResponse;
exports.default = callGQConsole;

var _abortController = _interopRequireDefault(require("abort-controller"));

var _url = require("url");

var _crypto = require("crypto");

var _nodeFetch = _interopRequireWildcard(require("node-fetch"));

var _logger = _interopRequireDefault(require("../../logger"));

var _error = require("../error");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getSignature(secretKey, url) {
  return (0, _crypto.createHmac)('sha1', secretKey).update(`${url.pathname}${url.search}`).digest('base64');
}

function getRequest(conf, method, command, {
  version = 'api/v1',
  auth = false,
  params = {},
  data
}) {
  const url = new _url.URL(conf.url);
  const headers = new _nodeFetch.Headers();
  const init = {
    method,
    headers
  };

  if (conf.timeout) {
    const controller = new _abortController.default();
    init.signal = controller.signal;
    setTimeout(() => controller.abort(), conf.timeout);
  }

  url.pathname = version ? `/${version}/${command}/` : `/${command}/`;

  if (auth && conf.apiKey && conf.secretKey) {
    const {
      apiKey: username,
      secretKey: password
    } = conf;
    const encodedPassword = Buffer.from(password).toString('base64');
    const encoded = Buffer.from(`${username}:${encodedPassword}`).toString('base64');
    headers.append('Content-Type', 'application/json; charset=utf-8');
    headers.append('Authorization', `Basic ${encoded}`); // $FlowFixMe

    return new _nodeFetch.Request(url.href, init);
  }

  url.searchParams.append('format', 'json');

  if (conf.apiKey) {
    url.searchParams.append('apikey', conf.apiKey);
  }

  Object.entries(params).forEach(([key, value]) => {
    if (value === null || typeof value === 'undefined' || typeof value.toString !== 'function') {
      throw new TypeError(`Value for param "${key}" must be string compatible`);
    } // $FlowFixMe


    url.searchParams.append(key, value.toString());
  });
  url.searchParams.sort();

  if (conf.secretKey) {
    url.searchParams.append('signature', getSignature(conf.secretKey, url));
  }

  _logger.default.silly(url.href);

  if (data) {
    init.body = JSON.stringify(data);
    headers.append('Content-Type', 'application/json; charset=utf-8');
  } // $FlowFixMe


  return new _nodeFetch.Request(url.href, init);
}

async function handleResponse(response, {
  metrics,
  method,
  command,
  start
} = {}) {
  _logger.default.silly(`GQ-console response=${JSON.stringify(response)}`);

  if (metrics) {
    metrics.histogram.labels(response.status, method, command.split('/').filter(segment => Number.isNaN(parseInt(segment, 10))).join('/')).observe(Date.now() - start);
  }

  const contentType = response.headers.get('Content-Type');
  let result;

  if (contentType) {
    if (contentType.includes('application/json')) {
      result = await response.json();
    } else {
      result = await response.text();
    }
  } // $FlowFixMe


  _logger.default.debug(result);

  if (!response.ok) {
    let message = response.statusText;

    if (typeof result === 'string') {
      message = result;
    } else if (result && !(result instanceof ArrayBuffer)) {
      message = result.message || result.error || result.error_text || result.detail;
    }

    throw new _error.ServiceError(response.status, message, result);
  }

  return result;
}

function callGQConsole(conf, method, command, options = {}) {
  _logger.default.debug(`GQConsole method=${method}`);

  _logger.default.debug(`GQConsole command=${command}`);

  _logger.default.debug(`GQConsole options=${JSON.stringify(options)}`);

  const start = Date.now();
  const request = getRequest(conf, method, command, options);
  return (0, _nodeFetch.default)(request).then(result => handleResponse(result, {
    metrics: conf.metrics,
    method,
    command,
    start
  })).catch(e => {
    if (e.type === 'aborted') {
      throw new _error.ServiceError(408, `Request timeout: Timed out after ${conf.timeout || ''} milliseconds`, e);
    }

    throw e;
  });
}