//
// Copyright 2015-2016 Greenqloud ehf - www.qstack.com
//

'use strict';

var crypto = require('crypto');

var SEPARATOR = ':';


exports.Version = function(version) {
    if (!version || version === "V1") {
        return {
            name: 'V1',
            algorithm: 'AES',
            bits: 256,
            mode: 'CBC',
            padding: 'PKCS7Padding',
            key_derivation_algorithm: 'PBKDF2WithHmacSHA1',
            iterations: 20000
        };
    }
    throw TypeError('Invalid Version:', version);
};


// SecretsHandler
exports.SecretsHandler = function(block_size /* = 16 */) {
    if (! block_size) {
        block_size = 16;
    }
    this.version = exports.Version();
    this.block_size = block_size;
};


exports.SecretsHandler.prototype.get_derived_key = function(passphrase, salt, iterations) {
    var length = Math.floor(this.version.bits / 8);
    return crypto.pbkdf2Sync(passphrase, salt, iterations, length, 'SHA1');
};


exports.SecretsHandler.prototype.decrypt = function(passphrase, text) {
    var parsed = this.parse(text);
    var key = this.get_derived_key(passphrase, parsed.salt, this.version.iterations);
    var algorithm = this.version.algorithm + this.version.bits;
    var decipher = crypto.createDecipheriv(algorithm, key, parsed.iv);
    var decrypted = decipher.update(parsed.enc_text);
    decrypted += decipher.final('utf8');
    return decrypted;
};


exports.SecretsHandler.prototype.padding = function(text) {
    // Assuming PKCS#7 since that is the only one we support currently.
    var c = this.block_size  - (Buffer(text, 'utf8').length % this.block_size);
    var result = '';
    for (var i = 0; i < c; i++) {
        result += String.fromCharCode(c);
    }
    return result;
};


exports.SecretsHandler.prototype.encrypt = function(passphrase, text) {
    var salt = crypto.randomBytes(8);
    var key = this.get_derived_key(passphrase, salt, this.version.iterations);
    var iv = crypto.randomBytes(16);
    var algorithm = this.version.algorithm + this.version.bits;
    var cipher = crypto.createCipheriv(algorithm, key, iv);
    var enc_text = cipher.update(text, 'utf8', 'binary');
    // PKCS#7 padding implementation (not required):
    //cipher.setAutoPadding(false);
    //enc_text += cipher.update(this.padding(text), 'utf8', 'binary');
    enc_text += cipher.final('binary');
    var enc_text_base64 = (new Buffer(enc_text, 'binary')).toString('base64');
    var salt_base64 = salt.toString('base64');
    var iv_base64 = iv.toString('base64');
    return this.version.name + SEPARATOR + salt_base64 + SEPARATOR + iv_base64 + SEPARATOR + enc_text_base64;
};


exports.SecretsHandler.prototype.parse = function(text) {
    var components = text.split(SEPARATOR);
    if (components.length !== 4) {
        throw TypeError('Input has incorrect number of fields');
    }
    var version = components[0];
    if (version !== this.version.name) {
        throw TypeError('Version is not supported:', version);
    }
    var salt = new Buffer(components[1], 'base64');
    var iv = new Buffer(components[2], 'base64');
    var enc_text =  new Buffer(components[3], 'base64');
    if (salt.length === 0 || iv.length === 0 || enc_text.length === 0) {
        throw TypeError('Invalid base64 encoding:');
    }
    return {
        version: version,
        salt: salt,
        iv: iv,
        enc_text: enc_text
    };
};
