'use strict';

var encryption = require('./encryption.js');

var crypto = require('crypto');

var HASH_FUNCTION = 'sha256';
var M1_PREFIX = 'M1';
var M1_SEP = '.';
var E1_PREFIX = 'V1';
var E1_SEP = ':';


// Creates encrypted url safe messages with timestamp (for expiration support).
// For reference encryption.js message is of the form:
//      V1.base64(salt).base64(iv).base64(encText)
//      encText: encrypt(text)
//
// Format of the secure message:
//      M1.hmac().hex(salt).hex(iv).hex(date).hex(encText)
//      hmac(): hash(key + salt + iv + date + encText)
//
// It is up to the receiver to validate the date and decide on expiry but time could be in the
// future if clocks are not correctly set (and messaging is happening between two machines).
exports.createSecureMessage = function(key, text, date /* undefined ==> current time */) {
    var crypter = new encryption.SecretsHandler();
    var encrypted = crypter.encrypt(key, text);
    var safeDate = exports.validDate(date);
    var parsed = crypter.parse(encrypted);
    var hmacHex = exports.calculateHmac(key, parsed.salt, parsed.iv, safeDate, parsed.enc_text);
    var saltHex = parsed.salt.toString('hex');
    var ivHex = parsed.iv.toString('hex');
    var safeDateHex = (new Buffer(safeDate)).toString('hex');
    var payloadHex = parsed.enc_text.toString('hex');
    return M1_PREFIX + M1_SEP + hmacHex + M1_SEP + saltHex + M1_SEP + ivHex + M1_SEP + safeDateHex + M1_SEP + payloadHex;
};


// Checks messages for validity - returns decrypted message if everything is ok.
// if message is malformed throws exception
// if date range does not match returns empty string: ''
exports.validateSecureMessage = function(key, message, date, milliSecPast, milliSecFuture) {
    var parsed = exports.parse(message);
    var safeDate = exports.validDate(date);
    var calculatedHmac = exports.calculateHmac(key, parsed.salt, parsed.iv, parsed.date, parsed.encText);
    if (calculatedHmac !== parsed.hmac.toString('hex')) {
        throw TypeError('HMAC did not match!');
    }
    if (!exports.dateInRange(parsed.date.toString('utf8'), safeDate, milliSecPast, milliSecFuture)) {
        return '';
    }
    var crypter = new encryption.SecretsHandler();
    var encryptionFormat = E1_PREFIX + E1_SEP +
        parsed.salt.toString('base64') + E1_SEP +
        parsed.iv.toString('base64') + E1_SEP +
        parsed.encText.toString('base64');
    return crypter.decrypt(key,encryptionFormat);
};


exports.parse = function(secureMessage) {
    var components = secureMessage.split(M1_SEP);
    if (components.length !== 6) {
        throw TypeError('Input has incorrect number of fields');
    }
    var version = components[0].toUpperCase();
    if (version !== M1_PREFIX) {
        throw TypeError('Version is not supported:', version);
    }
    var hmac = new Buffer(components[1], 'hex', 'binary');
    var salt = new Buffer(components[2], 'hex', 'binary');
    var iv = new Buffer(components[3], 'hex', 'binary');
    var date = new Buffer(components[4], 'hex', 'binary');
    var encText =  new Buffer(components[5], 'hex', 'binary');
    
    return {
        version: version,
        hmac: hmac,
        salt: salt,
        iv: iv,
        date: date,
        encText: encText
    };
};


exports.calculateHmac = function(key, salt, iv, date, encText) {
    var hmac = crypto.createHmac(HASH_FUNCTION, key);
    hmac.update(salt, 'binary');
    hmac.update(iv, 'binary');
    var dateBuffer = new Buffer(date, 'utf8');
    hmac.update(dateBuffer, 'binary');
    hmac.update(encText, 'binary');
    return hmac.digest('hex');
};


// returns 'true' iff (date2 - date1) is in the range milliSecPast ... milliSecFuture
// date2 can be undefined and is interpreted as 'now'
exports.dateInRange = function(date1, date2, milliSecPast, milliSecFuture) {
    var d1 = Date.parse(date1);
    var d2 = Date.parse(exports.validDate(date2));
    if (!isNaN(d1) && !isNaN(d2)) {
        var diff = d2 - d1;
        if (diff <= milliSecPast && diff >= -milliSecFuture) {
            return true;
        }
    }
    return false;
};


// returns date if defined or now in ISO 8601 format if not.
exports.validDate = function(date) {
    if (date) {
        return date;
    }
    else {
        return (new Date()).toISOString();
    }
};
